// ***************************************************************************
// ***************************************************************************
// Copyright 2011(c) Analog Devices, Inc.
// 
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//     - Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     - Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in
//       the documentation and/or other materials provided with the
//       distribution.
//     - Neither the name of Analog Devices, Inc. nor the names of its
//       contributors may be used to endorse or promote products derived
//       from this software without specific prior written permission.
//     - The use of this software may or may not infringe the patent rights
//       of one or more patent holders.  This license does not release you
//       from the requirement that you obtain separate licenses from these
//       patent holders to use this software.
//     - Use of the software either in source or binary form, must be run
//       on or directly connected to an Analog Devices Inc. component.
//    
// THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED.
//
// IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, INTELLECTUAL PROPERTY
// RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
// BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
// THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// ***************************************************************************
// ***************************************************************************
// ***************************************************************************
// ***************************************************************************
// This is the LVDS/DDR interface

`timescale 1ns/100ps

module cf_adc_if (

  // jesd interface (ref_clk is the sample clock, rx_clk is ref_clk/2)

  rx_clk,
  rx_sof,
  rx_eof,
  rx_ferr0,
  rx_ferr1,
  rx_ferr2,
  rx_ferr3,
  rx_data0,
  rx_data1,
  rx_data2,
  rx_data3,
  ref_clk,

  // adc data output

  adc_valid,
  adc_data,

  // adc status

  adc_or,
  adc_pn_oos,
  adc_pn_err,

  // processor interface

  up_pn_type,
  up_data_sel,

  // debug interface (chipscope)

  debug_data,
  debug_trigger);

  // jesd interface (ref_clk is the sample clock, rx_clk is ref_clk/2)

  input           rx_clk;
  input   [ 3:0]  rx_sof;
  input   [ 3:0]  rx_eof;
  input   [ 3:0]  rx_ferr0;
  input   [ 3:0]  rx_ferr1;
  input   [ 3:0]  rx_ferr2;
  input   [ 3:0]  rx_ferr3;
  input   [31:0]  rx_data0;
  input   [31:0]  rx_data1;
  input   [31:0]  rx_data2;
  input   [31:0]  rx_data3;
  input           ref_clk;

  // adc data output

  output          adc_valid;
  output  [63:0]  adc_data;

  // adc status

  output  [ 7:0]  adc_or;
  output  [ 7:0]  adc_pn_oos;
  output  [ 7:0]  adc_pn_err;

  // processor interface

  input   [ 7:0]  up_pn_type;
  input           up_data_sel;

  // debug interface (chipscope)

  output [151:0]  debug_data;
  output  [ 7:0]  debug_trigger;

  reg     [ 6:0]  adc_raddr = 'd0;
  reg    [127:0]  adc_rdata = 'd0;
  reg             adc_valid = 'd0;
  reg     [63:0]  adc_data = 'd0;
  reg     [ 5:0]  adc_waddr = 'd0;
  reg     [ 3:0]  adc_sof = 'd0;
  reg     [ 3:0]  adc_eof = 'd0;
  reg     [15:0]  adc_ferr = 'd0;
  reg     [15:0]  adc_data0 = 'd0;
  reg     [15:0]  adc_data1 = 'd0;
  reg     [15:0]  adc_data2 = 'd0;
  reg     [15:0]  adc_data3 = 'd0;
  reg     [15:0]  adc_data4 = 'd0;
  reg     [15:0]  adc_data5 = 'd0;
  reg     [15:0]  adc_data6 = 'd0;
  reg     [15:0]  adc_data7 = 'd0;
  reg     [ 7:0]  adc_or = 'd0;

  wire   [127:0]  adc_rdata_s;

  // debug signals

  assign debug_trigger = {adc_sof, adc_eof};
  
  assign debug_data[151:148] = adc_sof;
  assign debug_data[147:144] = adc_eof;
  assign debug_data[143:128] = adc_ferr;
  assign debug_data[127:  0] = {adc_data7, adc_data6, adc_data5, adc_data4,
    adc_data3, adc_data2, adc_data1, adc_data0};

  // adc data output from this module is at 2x sampling frequency. because we need
  // to write 64bits at every clock- from 128 bits of incoming data

  always @(posedge ref_clk) begin
    adc_raddr <= adc_raddr + 1'b1;
    adc_rdata <= adc_rdata_s;
    adc_valid <= 1'b1;
    adc_data <= (adc_raddr[0] == 1'b0) ? adc_rdata[63:0] : adc_rdata[127:64];
  end

  // the gtx data is always at the sampling frequency (since we are 32x4 wide- 128 = 8x16)
  // so we could directly use the data for the PN monitors

  always @(posedge rx_clk) begin
    adc_waddr <= adc_waddr + 1'b1;
    adc_sof <= rx_sof;
    adc_eof <= rx_eof;
    adc_ferr <= {rx_ferr3, rx_ferr2, rx_ferr1, rx_ferr0};
    adc_data0 <= {rx_data0[ 7: 0], rx_data0[15: 8]};
    adc_data1 <= {rx_data0[23:16], rx_data0[31:24]};
    adc_data2 <= {rx_data1[ 7: 0], rx_data1[15: 8]};
    adc_data3 <= {rx_data1[23:16], rx_data1[31:24]};
    adc_data4 <= {rx_data2[ 7: 0], rx_data2[15: 8]};
    adc_data5 <= {rx_data2[23:16], rx_data2[31:24]};
    adc_data6 <= {rx_data3[ 7: 0], rx_data3[15: 8]};
    adc_data7 <= {rx_data3[23:16], rx_data3[31:24]};
    adc_or <= 'd0;
  end

  // pn monitors

  cf_pnmon i_pnmon7 (
    .adc_clk (rx_clk),
    .adc_data (adc_data7),
    .adc_pn_oos (adc_pn_oos[7]),
    .adc_pn_err (adc_pn_err[7]),
    .up_pn_type (up_pn_type[7]));

  cf_pnmon i_pnmon6 (
    .adc_clk (rx_clk),
    .adc_data (adc_data6),
    .adc_pn_oos (adc_pn_oos[6]),
    .adc_pn_err (adc_pn_err[6]),
    .up_pn_type (up_pn_type[6]));

  cf_pnmon i_pnmon5 (
    .adc_clk (rx_clk),
    .adc_data (adc_data5),
    .adc_pn_oos (adc_pn_oos[5]),
    .adc_pn_err (adc_pn_err[5]),
    .up_pn_type (up_pn_type[5]));

  cf_pnmon i_pnmon4 (
    .adc_clk (rx_clk),
    .adc_data (adc_data4),
    .adc_pn_oos (adc_pn_oos[4]),
    .adc_pn_err (adc_pn_err[4]),
    .up_pn_type (up_pn_type[4]));

  cf_pnmon i_pnmon3 (
    .adc_clk (rx_clk),
    .adc_data (adc_data3),
    .adc_pn_oos (adc_pn_oos[3]),
    .adc_pn_err (adc_pn_err[3]),
    .up_pn_type (up_pn_type[3]));

  cf_pnmon i_pnmon2 (
    .adc_clk (rx_clk),
    .adc_data (adc_data2),
    .adc_pn_oos (adc_pn_oos[2]),
    .adc_pn_err (adc_pn_err[2]),
    .up_pn_type (up_pn_type[2]));

  cf_pnmon i_pnmon1 (
    .adc_clk (rx_clk),
    .adc_data (adc_data1),
    .adc_pn_oos (adc_pn_oos[1]),
    .adc_pn_err (adc_pn_err[1]),
    .up_pn_type (up_pn_type[1]));

  cf_pnmon i_pnmon0 (
    .adc_clk (rx_clk),
    .adc_data (adc_data0),
    .adc_pn_oos (adc_pn_oos[0]),
    .adc_pn_err (adc_pn_err[0]),
    .up_pn_type (up_pn_type[0]));

  // transfer data to the reference clock side (2x)

  cf_mem #(.DW(128), .AW(6)) i_mem (
    .clka (rx_clk),
    .wea (1'b1),
    .addra (adc_waddr),
    .dina ({adc_data7, adc_data6, adc_data5, adc_data4,
      adc_data3, adc_data2, adc_data1, adc_data0}),
    .clkb (ref_clk),
    .addrb (adc_raddr[6:1]),
    .doutb (adc_rdata_s));

endmodule

// ***************************************************************************
// ***************************************************************************

